import os
import glob
import yaml
import shutil
import hashlib
import huggingface_hub as hh
from .settings import config
folder = os.path.join(config['axiom_path'], 'obj')
if not os.path.isdir(folder):
    os.makedirs(folder)

folder = os.path.join(config['axiom_path'], 'info')
if not os.path.isdir(folder):
    os.mkdir(folder)

token = open(os.path.join(config['home'], '.huggingface/token')).read()


def upload_to_server(md5):
    obj_file = os.path.join(config['axiom_path'], 'obj', md5)
    info_file = os.path.join(config['axiom_path'], 'info', md5+'.yaml')
    hh.upload_file(
        path_or_fileobj=obj_file,
        path_in_repo=md5,
        repo_id='reaxiom/obj',
        repo_type="dataset",
        token=token
    )
    hh.upload_file(
        path_or_fileobj=info_file,
        path_in_repo=md5+'.yaml',
        repo_id='reaxiom/info',
        repo_type="dataset",
        token=token
    )

def download_obj_from_server(md5):
    cache_dir = config['huggingface_cache']
    repo_id = 'reaxiom/obj'
    hh.hf_hub_download(
        repo_id=repo_id,
        filename=md5,
        repo_type="dataset",
        cache_dir=cache_dir,
        token=token
    )
    sha = hh.dataset_info(repo_id).sha
    obj_folder = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id.replace('/','--'), 'snapshots', sha)
    extract_folder = os.path.join(config['axiom_cache'], md5, 'obj')
    if not os.path.isdir(extract_folder):
        os.makedirs(extract_folder)
        md5_full_path = os.path.join(obj_folder, md5)
        cmd = 'tar zxf %s -C %s' % (md5_full_path, extract_folder)
        print(cmd)
        os.system(cmd)

def get_uuid_by_given_name_in_folder(folder, given_name):
    fs = glob.glob(os.path.join(folder, '*.yaml'))
    ys = [yaml.safe_load(open(f).read()) for f in fs]
    uuids = [os.path.split(fs[i])[-1][:-5] for i in range(len(fs)) if ys[i]['given_name'] == given_name]
    if len(uuids) >= 1:
        return uuids[0]
    else:
        return None

def get_uuid_by_given_name_from_server(given_name):
    cache_dir = config['huggingface_cache']
    repo_id = 'reaxiom/info'
    cache_dir = config['huggingface_cache']
    hh.snapshot_download(repo_id, repo_type='dataset', cache_dir=cache_dir, token=True)
    sha = hh.dataset_info(repo_id).sha
    info_folder = os.path.join(config['huggingface_cache'], 'datasets--'+repo_id.replace('/','--'), 'snapshots', sha)
    md5 = get_uuid_by_given_name_in_folder(info_folder, given_name)
    print('md5', md5, 'found at:', info_folder)
    axiom_cache_info_folder = os.path.join(config['axiom_cache'], md5)
    if not os.path.isdir(axiom_cache_info_folder):
        os.mkdir(axiom_cache_info_folder)
    axiom_cache_info_file = os.path.join(axiom_cache_info_folder, 'info.yaml')
    remote_cache_info_file = os.path.join(info_folder, md5+'.yaml')
    axiom_info_file = os.path.join(config['axiom_path'], 'info', md5+'.yaml')
    info_content = open(remote_cache_info_file).read()
    open(axiom_cache_info_file, 'w').write(info_content)
    open(axiom_info_file, 'w').write(info_content)
    return md5

def get_uuid_by_given_name(given_name):
    axiom_path = config['axiom_path']
    md5 = get_uuid_by_given_name_in_folder(os.path.join(axiom_path, 'info'), given_name)
    if md5 is not None:
        return md5
    print('not found in local path: %s, checking with huggingface' % axiom_path)
    md5 = get_uuid_by_given_name_from_server(given_name)
    if md5 is None:
        print('get_uuid_by_given_name return None')
    return md5

def get_folder_md5(folder):
    md5 = hashlib.md5()
    for subdir, _, files in os.walk(folder):
        for f in files:
            full_name = os.path.join(subdir, f)
            with open(full_name, 'rb') as f:
                for chunk in iter(lambda: f.read(4096), b""):
                    md5.update(chunk)
    return md5.hexdigest()


def add_obj_common(folder):
    info_file = os.path.join(os.path.split(folder)[0], 'info.yaml')
    if not os.path.exists(info_file):
        if info_file == 'info.yaml':
            info_file = os.path.join('.', info_file)
        print("make sure %s exists" % info_file)
        return None
    obj_folder = os.path.split(folder)[-1]
    md5 = os.path.split(os.path.split(folder)[0])[-1]
    cmd = 'cd '+folder+'&& tar zfc ../'+obj_folder+'.tar.gz ./'
    print(cmd)
    os.system(cmd)

    shutil.move(folder+'.tar.gz', os.path.join(config['axiom_path'], 'obj', md5))
    f = os.path.join(config['axiom_path'], 'info', md5+'.yaml')
    print('f', f)
    open(f,'w').write(open(info_file).read())




import huggingface_hub as hh

import huggingface_hub as hh
repo_id = 'dijunluo/axiom3'
hh.create_repo(repo_id, repo_type='dataset', private=False)
ff = hh.upload_file(
    path_or_fileobj='/home/aistudio/t5.py',
    path_in_repo="obj/xxx",
    repo_id=repo_id,
    repo_type="dataset",
)

hf_hub_download('dijunluo/axiom', 'obj/xxx')
print(ff)
## curl https://huggingface.co/datasets/dijunluo/axiom3/raw/main/obj/xxx


import os
__home = os.environ['HOME']
config = {
    'home': __home,
    'axiom_path': os.path.join(__home, '.axiom'),
    'axiom_cache': os.path.join(__home, '.axiom', 'cache'),
    'huggingface_cache': os.path.join(__home, '.axiom', 'huggingface_cache'),
}



import os
import sys
import yaml
import uuid
import time
import shutil
import datetime
import importlib
from .settings import config as _config
from functools import partial
from .common import get_uuid_by_given_name, get_folder_md5, add_obj_common, download_obj_from_server


def get_function_by_md5(md5):
    if md5 is not None:
        function_folder = os.path.join(_config['axiom_cache'], md5, 'obj')
        if not os.path.isdir(function_folder):
            download_obj_from_server(md5)
        sys.path.append(function_folder)
        print('function_folder', function_folder)
        run_file = 'main'
        x = importlib.import_module(run_file)
        print(x)
        return x.main
    sys.exit(0)

def get_function_by_given_name(given_name):
    md5 = get_uuid_by_given_name(given_name)
    return get_function_by_md5(md5)

def the_func(config={}, in_folders=[], given_names=[], out_folders=None, func_name=''):
    md5 = get_uuid_by_given_name(func_name)
    f = get_function_by_md5(md5)
    if out_folders is None:
        out_folders = []
        n = len(given_names)
        obj_folders = []
        for i in range(n):
            random_id = str(uuid.uuid4())
            out_folder = os.path.join(_config['axiom_cache'], random_id)
            obj_folder = os.path.join(out_folder, 'obj')
            obj_folders.append(obj_folder)
            print('making dir', obj_folder)
            os.makedirs(obj_folder)
            out_folders.append(out_folder)
    function_info = yaml.safe_load(open(os.path.join(_config['axiom_path'], 'info', md5+'.yaml')))
    if function_info['axiom_name'] == '__AXIOM__':
        function_info['axiom_name'] = md5
    print('function_info', function_info)
    t_begin = time.time()
    time_begin = str(datetime.datetime.now())
    f(config, in_folders, given_names, obj_folders)
    time_end = str(datetime.datetime.now())
    t_end = time.time()
    in_axiom_names = []
    for i in range(len(in_folders)):
        in_folder = in_folders[i]
        d = yaml.safe_load(open(os.path.join(os.path.split(in_folder)[0], 'info.yaml')).read())
        if d['axiom_name'] == '__AXIOM__':
            in_axiom_names.append(os.path.split(os.path.split(in_folder)[0])[-1])
        else:
            in_axiom_names.append(d['axiom_name'])


    print('in_axiom_names', in_axiom_names)
    info = {}
    uname = str(os.uname()).replace("'", '"').replace('\n', '')
    n_out = len(out_folders)
    for i in range(n_out):
        out_folder = out_folders[i]
        given_name = given_names[i]
        the_type = function_info['output'][i]
        description = 'data from function '+func_name
        info['type'] = the_type
        info['given_name'] = given_name
        info['axiom_name'] = {'function': function_info['axiom_name'], 'in_data': in_axiom_names, 'out': '%d/%d' % (i, n_out)}
        info['description'] = description
        info['run_begin_time'] = time_begin
        info['run_end_time'] = time_end
        info['running_time'] = t_end - t_begin
        info['uname'] = uname
        info_file = os.path.join(out_folder,'info.yaml')
        yaml.safe_dump(info, open(info_file, 'w'))
    for out_folder in out_folders:
        md5 = get_folder_md5(os.path.join(out_folder, 'obj')) # md5 is the one with obj folder
        print('out_folder',out_folder)
        md5_folder = os.path.join(os.path.split(out_folder)[0], md5)
        print('md5_folder', md5_folder)
        shutil.move(out_folder, md5_folder)
        add_obj_common(os.path.join(md5_folder, 'obj'))
    return out_folders




def main(func_name):
    print('axiom import', func_name)
    return partial(the_func, func_name=func_name)



import os
from .settings import config
from .common import get_uuid_by_given_name, download_obj_from_server
def main(db, by='uuid'):
    md5 = None
    if by == 'given_name':
        md5 = get_uuid_by_given_name(db)
    elif by == 'uuid':
        md5 = db
    else:
        print('by type not found:', by)
    local_folder = os.path.join(config['axiom_cache'], md5, 'obj')
    if not os.path.isdir(local_folder):
        download_obj_from_server(md5)
    return local_folder



import os
import sys
import uuid
import yaml
import json
import glob
import time
import socket
import shutil
import hashlib
import base64
import inspect
import importlib
import pandas as pd
import axiom.common
from itertools import chain
folder_me = os.path.split(__file__)[0]
from axiom.settings import config

def add_obj(config, folder, remove=True):
    info_file = os.path.join(os.path.split(folder)[0], 'info.yaml')
    if not os.path.exists(info_file):
        if info_file == 'info.yaml':
            info_file = os.path.join('.', info_file)
        print("make sure %s exists" % info_file)
        return None
    md5 = axiom.common.get_folder_md5(folder)
    cmd = 'cd '+folder+'&& tar zfc ../'+os.path.split(folder)[-1]+'.tar.gz ./'
    print(cmd)
    os.system(cmd)

    os.rename(folder+'.tar.gz', os.path.join(config['axiom_path'], 'obj', md5))

    f = os.path.join(config['axiom_path'], 'info', md5+'.yaml')
    open(f,'w').write(open(info_file).read())
    import axiom.settings as settings
    if remove:
        shutil.move(os.path.split(folder)[0], os.path.join(settings.config['axiom_cache'], md5))
    else:
        shutil.copytree(os.path.split(folder)[0], os.path.join(settings.config['axiom_cache'], md5))
    return md5



if __name__ == '__main__':
    if len(sys.argv) == 1:
        lines = open(__file__).read().split('\n')
        lines = [line for line in lines if line.find('if sys.argv[1]') > 0][1:]
        cmds = [line.split("'")[1].split("'")[0] for line in lines]
        comments = [line.split("#")[-1] for line in lines]
        usages = [cmds[i]+': '+comments[i] for i in range(len(cmds))]
        print('\n'.join(usages))
        sys.exit()        

    if sys.argv[1] == 'info': # show basic info of the project
        print('my file name', __file__)
        print('config', config)

    def compile(txt):
        from axiom.settings import config
        folder = str(uuid.uuid4())
        os.makedirs(os.path.join(folder, 'obj'))
        me, code_txt = txt.split('##__AXIOM_INFO_END__\n')
        code_txt = code_txt.split('##__AXIOM_FUNCTION_END__')[0]
        print('me'.center(50,'='))
        print(me)
        print('code'.center(50,'='))
        print(code_txt)
        open(os.path.join(folder, 'obj', 'main.py'),'w').write(code_txt)
        info_file = os.path.join(folder, 'info.yaml')
        yaml.safe_dump(json.loads(me), open(info_file, 'w'))
        print("add_obj(os.path.join(folder, 'obj'), remove=True)", folder)
        add_obj(config, os.path.join(folder, 'obj'), remove=True)

    if sys.argv[1] == 'compile':
        txt = open(sys.argv[2]).read()
        blocks = txt.split('##__AXIOM_BLOCK_BEGIN__')[1:]
        blocks = [b.split('##__AXIOM_BLOCK_END__')[0] for b in blocks]
        for b in blocks:
            compile(b)

    if sys.argv[1] == 'what': # show what can we show
        fs = glob.glob(os.path.join(config['axiom'], 'info', '*'))
        yamls = [yaml.safe_load(open(f).read()) for f in fs]
        if len(sys.argv) == 2:
            keys = [list(y.keys()) for y in yamls]
            print(list(set(list(chain.from_iterable(keys)))))
        if len(sys.argv) == 3:
            obj_properties = []
            for y in yamls:
                x = y[sys.argv[2]]
                if isinstance(x, list):
                    obj_properties += x
                else:
                    obj_properties.append(x)

            obj_properties = sorted(list(set(obj_properties)))
            print('\n'.join(obj_properties))

    if sys.argv[1] == 'template': # axiom template report_matrix, add a function template at the current folder
        func_name = sys.argv[2]
        os.mkdir(func_name)
        open(os.path.join(func_name, '__main.py'), 'w').write(template_main)
        open(os.path.join(func_name, 'info.yaml'), 'w').write(template_info)
        open(os.path.join(func_name, '__INCLUDE__'), 'w').write('external folder')
        open(os.path.join(func_name, 'config.yaml'), 'w').write('{}')

    def list_axiom(axiom_path):
        fs = glob.glob(os.path.join(axiom_path, '*.yaml'))
        ys = [yaml.safe_load(open(f).read()) for f in fs]
        md5s = [os.path.split(f)[-1][:-5] for f in fs]
        columns = ['given_name', 'type', 'description']
        d = {}
        for i in range(len(fs)):
            d[md5s[i]] = [ys[i]['given_name'], ys[i]['type'], ys[i]['description']]
        df = pd.DataFrame.from_dict(d, orient='index', columns=columns)
        print(df)

    if sys.argv[1] == 'ls': # show the axiom.obj
        if len(sys.argv) == 3:
            axiom_path = sys.argv[2]
        else:
            axiom_path = config['axiom_path']
        list_axiom(os.path.join(axiom_path, 'info'))

    if sys.argv[1] == 'lsb': # list all the objects at huggingface axiom/info
        list_axiom(axiom.common.config['bypy_cache'])

    if sys.argv[1] == 'add': # add the current folder as the obj
        add_obj(sys.argv[2], remove=False)

    if sys.argv[1] == 'rm': # remove a md5 obj
        md5 = sys.argv[2]
        os.remove(os.path.join(config['axiom_path'], 'obj', md5))
        os.remove(os.path.join(config['axiom_path'], 'info', md5+'.yaml'))
        shutil.rmtree(os.path.join(config['axiom_cache'], md5))
    
    if sys.argv[1] == 'extract': # exting an object to the corrent folder
        md5 = sys.argv[2]
        f = os.path.join(config['axiom'], 'obj', md5)
        cmd = 'cp %s ./&& tar zfx %s&&rm %s' % (f, md5, md5)
        print(cmd)
        os.system(cmd)
    

    if sys.argv[1] == 'commit': # upload an md5 to bypy
        md5 = sys.argv[2]
        axiom.common.upload_to_server(md5)

    def run_python_block(txt):
        file_name = str(uuid.uuid4())
        open(file_name, 'w').write(txt)
        os.system('python '+file_name)
        os.remove(file_name)
        
    if sys.argv[1] == 't': # test a file with __AXIOM__BLOCK__BEGIN__ block, with test
        txt = open(sys.argv[2]).read()
        blocks = txt.split('##__AXIOM_BLOCK_BEGIN__')[1:]
        blocks = [b.split('##__AXIOM_BLOCK_END__')[0] for b in blocks]
        for b in blocks:
            run_python_block(b)

